package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStudentFinalScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentFinalScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.MarkingType;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentFinalScoreService;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description 考生最终分数实现类
 * @Author lideyin
 * @Date 2019/11/11 14:30
 * @Version 1.0
 */
@Service("examStudentFinalScoreService")
public class ExamStudentFinalScoreServiceImpl implements ExamStudentFinalScoreService {

    private static final Logger LOG = LoggerFactory.getLogger(ExamStudentFinalScoreServiceImpl.class);

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamStudentFinalScoreRepo examStudentFinalScoreRepo;

    @Override
    public ExamStudentFinalScoreEntity calcAndSaveFinalScore(Long examStudentId) {
        ExamStudentEntity examStudent = examStudentRepo.findByExamStudentId(examStudentId);

        String markingType = ExamCacheTransferHelper.getDefaultCachedExamProperty(examStudent.getExamId(),
                ExamProperties.MARKING_TYPE.name()).getValue();
        String examType = ExamCacheTransferHelper.getDefaultCachedExam(examStudent.getExamId()).getExamType();

        List<ExamRecordDataEntity> effectiveExamRecordDataList = examRecordDataRepo.findEffectiveDataByExamStudentIdList(Arrays.asList(examStudentId));
        if (CollectionUtils.isEmpty(effectiveExamRecordDataList)) {
            LOG.warn("calcAndSaveFinalScore examRecordDataList is empty, examStudentId = {}", examStudentId);
            return null;
        }

        //得到最终考试结果
        ExamScoreEntity finalEffectiveExamScore = getFinalEffectiveExamScore(effectiveExamRecordDataList, examType, markingType);
        if (finalEffectiveExamScore == null) {
            LOG.warn("calcAndSaveFinalScore finalExamScore is null, examStudentId = {}", examStudentId);
            return null;
        }

        //保存最终考试结果
        return saveExamStudentFinalScore(examStudentId, finalEffectiveExamScore);
    }


    @Override
    public ExamStudentFinalScoreEntity getFinalEffectiveExamScore(Long examStudentId) {
        ExamStudentFinalScoreEntity finalExamScore = examStudentFinalScoreRepo.findByExamStudentId(examStudentId);
        if (null != finalExamScore) {
            return finalExamScore;
        }

        //历史数据弥补：如果数据不存在，则重新计算并保存
        return this.calcAndSaveFinalScore(examStudentId);
    }

    private ExamStudentFinalScoreEntity saveExamStudentFinalScore(Long examStudentId, ExamScoreEntity finalEffectiveExamScore) {
        if (finalEffectiveExamScore == null) {
            return null;
        }

        ExamStudentFinalScoreEntity finalScoreEntity = examStudentFinalScoreRepo.findByExamStudentId(examStudentId);
        if (finalScoreEntity == null) {
            finalScoreEntity = new ExamStudentFinalScoreEntity();
        }

        finalScoreEntity.setExamRecordDataId(finalEffectiveExamScore.getExamRecordDataId());
        finalScoreEntity.setExamStudentId(examStudentId);
        finalScoreEntity.setObjectiveAccuracy(finalEffectiveExamScore.getObjectiveAccuracy());
        finalScoreEntity.setObjectiveScore(finalEffectiveExamScore.getObjectiveScore());
        finalScoreEntity.setSubjectiveScore(finalEffectiveExamScore.getSubjectiveScore());
        finalScoreEntity.setSuccPercent(finalEffectiveExamScore.getSuccPercent());
        finalScoreEntity.setTotalScore(finalEffectiveExamScore.getTotalScore());

        return examStudentFinalScoreRepo.save(finalScoreEntity);
    }

    /**
     * 计算得出最终有效成绩
     */
    private ExamScoreEntity getFinalEffectiveExamScore(List<ExamRecordDataEntity> examRecords, String examType, String markingType) {
        if (examRecords == null || examRecords.isEmpty()) {
            return null;
        }

        if (examType == null) {
            throw new StatusException("examType is null");
        }

        if (markingType == null) {
            throw new StatusException("markingType is null");
        }

        //取出有效记录的id
        List<Long> examRecordDataIds = examRecords.stream().map(ExamRecordDataEntity::getId).collect(Collectors.toList());

        List<ExamScoreEntity> examScoreList = examScoreRepo.findByExamRecordDataIdIn(examRecordDataIds);
        if (examScoreList == null || examScoreList.isEmpty()) {
            LOG.warn("examScoreList is empty, examRecordDataIds = {}", examRecordDataIds);
            return null;
        }

        //离线考试
        if (ExamType.OFFLINE.name().equals(examType)) {
            return examScoreList.get(0);
        }

        if (markingType.equals(MarkingType.ALL.name()) || markingType.equals(MarkingType.OBJECT_SCORE_MAX.name())) {
            //全部评阅规则或客观分最高规则:取总分最高
            List<ExamScoreEntity> examScores = examScoreList.stream()
                    .sorted((o1, o2) -> o2.getTotalScore().compareTo(o1.getTotalScore()))
                    .collect(Collectors.toList());
            return examScores.get(0);
        } else if (markingType.equals(MarkingType.LAST_SUBMIT.name())) {
            //最后一次提交规则:取最后一次的成绩
            List<ExamScoreEntity> examScores = examScoreList.stream()
                    .sorted((o1, o2) -> o2.getId().compareTo(o1.getId()))
                    .collect(Collectors.toList());
            return examScores.get(0);
        }

        return null;
    }

}
